from setuptools import setup, find_packages

setup(
	name = "ionstruct",
	version = "0.0.1",
	author = "Rahul Priyadarshan R",
	author_email = "rahulpriyadarshan19@gmail.com",
	description = "A Python package to generate visualizations of TEC measurements in the ionosphere.",
	packages = find_packages()
	
)
